/* This module implements and interface to the SLang KBI routines */
#include <stdio.h>
#include <slang.h>
   
SLANG_MODULE(kbi);

static unsigned int kbi_getkey (void)
{
   return SLang_getkey();
}

static int kbi_kp_getkey (void)
{
   return SLkp_getkey();
}

static int kbi_kp_init (void)
{
   SLtt_get_terminfo ();
   if (-1 == SLkp_init ())
     {	
	SLang_doerror ("SLkp_init failed.");
	exit (1);
     }
   return 0;
}

static void kbi_reset_tty (void)
{
   SLang_reset_tty();
}

static SLang_Intrin_Fun_Type Kbi_Intrinsics [] =
{
   MAKE_INTRINSIC_0("kbi_getkey", kbi_getkey, SLANG_UINT_TYPE),
   MAKE_INTRINSIC_0("kbi_kp_getkey", kbi_kp_getkey, SLANG_INT_TYPE),
   MAKE_INTRINSIC_0("kbi_kp_init", kbi_kp_init, SLANG_INT_TYPE),
   MAKE_INTRINSIC_0("kbi_reset_tty", kbi_reset_tty, SLANG_VOID_TYPE),
   SLANG_END_INTRIN_FUN_TABLE
};

static SLang_IConstant_Type Kbi_Constants [] =
{
   MAKE_ICONSTANT("KBI_KEY_ERR", SL_KEY_ERR),
   MAKE_ICONSTANT("KBI_KEY_UP", SL_KEY_UP),
   MAKE_ICONSTANT("KBI_KEY_DOWN", SL_KEY_DOWN),
   MAKE_ICONSTANT("KBI_KEY_LEFT", SL_KEY_LEFT),
   MAKE_ICONSTANT("KBI_KEY_RIGHT", SL_KEY_RIGHT),
   MAKE_ICONSTANT("KBI_KEY_PPAGE", SL_KEY_PPAGE),
   MAKE_ICONSTANT("KBI_KEY_NPAGE", SL_KEY_NPAGE),
   MAKE_ICONSTANT("KBI_KEY_HOME", SL_KEY_HOME),
   MAKE_ICONSTANT("KBI_KEY_END", SL_KEY_END),
   MAKE_ICONSTANT("KBI_KEY_BACKSPACE", SL_KEY_BACKSPACE),
   MAKE_ICONSTANT("KBI_KEY_ENTER", SL_KEY_ENTER),
   MAKE_ICONSTANT("KBI_KEY_IC", SL_KEY_IC),
   MAKE_ICONSTANT("KBI_KEY_DELETE", SL_KEY_DELETE),
   MAKE_ICONSTANT("KBI_KEY_F0", SL_KEY_F0),
   SLANG_END_ICONST_TABLE
};

int init_kbi_module_ns (char *ns_name)
{  
   int abort_char = 7;
   
   SLang_NameSpace_Type *ns = SLns_create_namespace (ns_name);
   if (ns == NULL)
     return -1;

   if ((-1 == SLns_add_intrin_fun_table (ns, Kbi_Intrinsics, "__KBI__"))
     || (-1 == SLns_add_iconstant_table (ns, Kbi_Constants, NULL)))
     return -1;
   
   if (-1 == SLang_init_tty (abort_char, 0, 1))
     {
   	fprintf (stderr, "Unable to initialize the terminal.\n");
   	exit (-1);
     }
   SLang_set_abort_signal (NULL);
   
   return 0;
}
  
int init_kbi_module (void)
{
   int abort_char = 7;

   if ((-1 == SLadd_intrin_fun_table (Kbi_Intrinsics, "__KBI__"))
     || (-1 == SLadd_iconstant_table (Kbi_Constants, NULL)))
     return -1;
   
   if (-1 == SLang_init_tty (abort_char, 0, 1))
     {
   	fprintf (stderr, "Unable to initialize the terminal.\n");
   	exit (-1);
     }
   SLang_set_abort_signal (NULL);
   
   return 0;
}

void deinit_kbi_module (void)
{
   SLang_reset_tty();
}
